@@ -2,7 +2,7 @@
  * $Id: connection.h,v 1.40 2005/04/12 19:34:35 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@
 #include <errno.h>
 #include <netdb.h>
 
-
 #ifdef HAVE_LIBSSL
 #include <openssl/crypto.h>
 #include <openssl/x509.h>
@@ -85,14 +84,15 @@ typedef struct connection {
 	SSL_CTX *ssl_ctx_h;
 	SSL *ssl_h;
 	int ssl_check_mode;
-	char *ssl_check_store;
 	X509 *cert;
 #endif
+	char *localip, *remoteip;
+	uint16_t localport, remoteport;
 } connection_t;
 
 connection_t *connection_new(char *dsthostname, int dstport, char *srchostname,
 		int srcport, int ssl, int ssl_check_mode,
-		char *ssl_check_store,int timeout);
+		char *ssl_check_store, char *ssl_client_certfile, int timeout);
 connection_t *listen_new(char *hostname, int port, int ssl);
 connection_t *accept_new(connection_t *cn);
 void connection_free(connection_t *cn);
