@@ -1345,15 +1345,29 @@ static int irc_353(struct link_server *server, struct line *line)
 	names = line->elemv[4];
 
 	while (*names) {
-		eon = names;
 		int ovmask = 0;
-		/* these should be exclusive */
-		if (*names == '@') {
-			names++;
-			ovmask |= NICKOP;
-		} else if (*names == '+') {
-			names++;
-			ovmask |= NICKVOICED;
+		int flagchars = 1;
+		/* some ircds (e.g. unreal) may display several flags for the
+                   same nick */
+		while (flagchars) {
+			switch (*names) {
+			case '@':
+				names++;
+				ovmask |= NICKOP;
+				break;
+			case '+':
+				names++;
+				ovmask |= NICKVOICED;
+				break;
+			case '%': /* unrealircd: halfop */
+			case '&': /* unrealircd: protect */
+			case '~': /* unrealircd: owner */
+				names++;
+				break;
+			default:
+				flagchars = 0;
+				break;
+			}
 		}
 		eon = names;
 		while (*eon && *eon != ' ')
@@ -1584,7 +1598,6 @@ static int irc_mode(struct link_server *server, struct line *line)
 			cur_arg++;
 			break;
 		case 'o':
-
 			if (cur_arg + 3 >= line->elemc)
 				return ERR_PROTOCOL;
 
@@ -1633,6 +1646,8 @@ static int irc_mode(struct link_server *server, struct line *line)
 		case 'h':
 		case 'H':
 		case 'e':
+		case 'q':
+		case 'I':
 			if (cur_arg + 3 >= line->elemc)
 				return ERR_PROTOCOL;
 			cur_arg++;
@@ -2121,8 +2136,7 @@ void oidentd_dump(bip_t *bip)
 	char tag_written = 0;
 
 	if (stat(bip->oidentdpath, &stats) == -1) {
-		if (errno == ENOENT) {
-			f = fopen(bip->oidentdpath, "w+");
+		if (errno == ENOENT && (f = fopen(bip->oidentdpath, "w+"))) {
 			fchmod(fileno(f), 0644);
 		} else {
 			mylog(LOG_WARN, "Can't open/create %s",
@@ -2130,6 +2144,7 @@ void oidentd_dump(bip_t *bip)
 			return;
 		}
 	} else {
+		/* strip previously autogenerated content */
 		char *content;
 		f = fopen(bip->oidentdpath, "r+");
 
@@ -2141,10 +2156,8 @@ void oidentd_dump(bip_t *bip)
 
 		content = (char *)malloc(stats.st_size + 1);
 
-		if (content == NULL){
+		if (content == NULL)
 			fatal("out of memory");
-			return;
-		}
 
 		if (fread(content, 1, stats.st_size, f) !=
 				(size_t)stats.st_size) {
