@@ -2,7 +2,7 @@
  * $Id: log.c,v 1.56 2005/04/21 06:58:50 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,23 +22,21 @@ extern char *conf_log_root;
 extern char *conf_log_format;
 extern int conf_log;
 
-int conf_memlog = 1;
-
-/* conf_always_backlog => conf_backlog_lines != 0 */
-extern int conf_backlog;
-extern int conf_backlog_lines;
-extern int conf_always_backlog;
-extern int conf_bl_msg_only;
-extern int conf_backlog_no_timestamp;
+extern FILE *conf_global_log_file;
 
 int log_set_backlog_offset(log_t *logdata, char *dest);
 static int _log_write(log_t *logdata, logfilegroup_t *lf, char *d, char *str);
 void logfile_free(logfile_t *lf);
 static char *_log_wrap(char *dest, char *line);
 
+#define BOLD_CHAR 0x02
+#define LAMESTRING "!bip@bip.bip.bip PRIVMSG "
+#define PMSG_ARROW " \002->\002 "
+
+
 /* TODO: change fatal("out of memory") to cleanup & return NULL */
 
-int check_dir(char *filename)
+int check_dir(char *filename, int is_fatal)
 {
 	int err;
 	struct stat statbuf;
@@ -47,15 +45,22 @@ int check_dir(char *filename)
 	if (err && errno == ENOENT) {
 		err = mkdir(filename, 0750);
 		if (err) {
+			if (is_fatal)
+				fatal("mkdir(%s) %s", filename,
+						strerror(errno));
 			mylog(LOG_ERROR, "mkdir(%s) %s", filename,
 					strerror(errno));
 			return 1;
 		}
 	} else if (err) {
+		if (is_fatal)
+			fatal("stat(%s) %s", filename, strerror(errno));
 		mylog(LOG_ERROR, "stat(%s) %s", filename,
 				strerror(errno));
 		return 1;
 	} else if (!(statbuf.st_mode & S_IFDIR)) {
+		if (is_fatal)
+			fatal("%s is not a directory", filename);
 		mylog(LOG_ERROR, "%s is not a directory", filename);
 		return 1;
 	}
@@ -88,7 +93,7 @@ int check_dir_r(char *dirname)
 		count += pos;
 		*(dir + count) = '\0';
 		mylog(LOG_DEBUGVERB,"check_dir_r: %s", dir);
-		if (check_dir(dir)) {
+		if (check_dir(dir, 0)) {
 			free(dir);
 			return 1;
 		}
@@ -146,7 +151,7 @@ char *log_build_filename(log_t *logdata, char *destination)
 	snprintf(month, 3, "%02d", now->tm_mon + 1);
 	snprintf(logfile, MAX_PATH_LEN, "%s/%s", conf_log_root,
 			conf_log_format);
-	replace_var(logfile, "%u", logdata->user, MAX_PATH_LEN);
+	replace_var(logfile, "%u", logdata->user->name, MAX_PATH_LEN);
 	replace_var(logfile, "%n", logdata->network, MAX_PATH_LEN);
 	replace_var(logfile, "%c", dest, MAX_PATH_LEN);
 	replace_var(logfile, "%Y", year, MAX_PATH_LEN);
@@ -257,7 +262,7 @@ static int log_add_file(log_t *logdata, char *destination, char *filename)
 		hash_insert(&logdata->logfgs, destination, lfg);
 	}
 
-	if (conf_memlog) {
+	if (!conf_log && logdata->user->backlog) {
 		if (!lfg->memlog)
 			lfg->memlog = list_new(NULL);
 	}
@@ -288,6 +293,7 @@ logfilegroup_t *log_find_file(log_t *logdata, char *destination)
 	char *filename = NULL;
 	time_t t;
 	struct tm *ltime;
+	struct link *l;
 
 	if (!ischannel(*destination))
 		destination = "privates";
@@ -312,6 +318,24 @@ logfilegroup_t *log_find_file(log_t *logdata, char *destination)
 		lfg = hash_get(&logdata->logfgs, destination);
 		if (!lfg)
 			fatal("internal log_find_file");
+		/* ok we are allocating a new lfg now, let's set it up for
+		 * backlogging if applicable */
+		if (!logdata->user)
+			fatal("log_find_file: no user associated to logdata");
+		if (!logdata->network)
+			fatal("log_find_file: no network id associated to "
+					"logdata");
+		l = hash_get(&logdata->user->connections, logdata->network);
+		if (!l)
+			fatal("log_beautify: no connection associated to "
+					"logdata");
+		struct chan_info *ci = hash_get(&l->chan_infos, destination);
+		if (ci && !ci->backlog) {
+			lfg->track_backlog = 0;
+		} else {
+			lfg->track_backlog = 1;
+		}
+
 		if (filename)
 			free(filename);
 		return lfg;
@@ -321,7 +345,7 @@ logfilegroup_t *log_find_file(log_t *logdata, char *destination)
 	time(&t);
 	ltime = localtime(&t);
 	lf = list_get_last(&lfg->file_group);
- 	if (ltime->tm_mday != lf->last_log.tm_mday) {
+	if (ltime->tm_mday != lf->last_log.tm_mday) {
 		logfile_t *oldlf;
 
 		/* day changed, we might want to rotate logfile */
@@ -344,7 +368,7 @@ logfilegroup_t *log_find_file(log_t *logdata, char *destination)
 		}
 		free(filename);
 
-		if (!conf_backlog) {
+		if (!logdata->user->backlog) {
 			/* remove oldlf from file_group */
 			if (list_remove_first(&lfg->file_group) != oldlf)
 				fatal("internal log_find_file 2");
@@ -362,6 +386,8 @@ logfilegroup_t *log_find_file(log_t *logdata, char *destination)
 
 /*
  * Da log routines
+ * There are a lot of snprintf's here without enforcing the last \0 in the
+ * buffer, but _log_write takes care of this for us.
  */
 void log_join(log_t *logdata, char *ircmask, char *channel)
 {
@@ -374,9 +400,14 @@ void log_join(log_t *logdata, char *ircmask, char *channel)
 void log_part(log_t *logdata, char *ircmask, char *channel,
 		char *message)
 {
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
+	if (message)
+		snprintf(logdata->buffer, LOGLINE_MAXLEN,
 			"%s -!- %s has left %s [%s]", timestamp(), ircmask,
 			channel, message);
+	else
+		snprintf(logdata->buffer, LOGLINE_MAXLEN,
+			"%s -!- %s has left %s", timestamp(), ircmask,
+			channel);
 	log_write(logdata, channel, logdata->buffer);
 }
 
@@ -424,7 +455,7 @@ static void _log_privmsg(log_t *logdata, char *ircmask, int src,
 		char *real_message = message;
 
 		if (*message == '+' || *message == '-')
-			real_message++; 
+			real_message++;
 
 		if (strncmp(real_message, "\001ACTION ", 8) != 0)
 			return;
@@ -603,17 +634,11 @@ void log_client_disconnected(log_t *logdata)
 	mylog(LOG_DEBUG, "A client disconnected");
 }
 
-void log_client_none_connected(log_t *logdata)
+void log_reinit_all(log_t *logdata)
 {
 	logfilegroup_t *lfg;
 	hash_iterator_t hi;
 
-	logdata->connected = 0;
-
-	if (conf_always_backlog)
-		return;
-
-
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
 			hash_it_next(&hi)) {
 		lfg = hash_it_item(&hi);
@@ -621,6 +646,16 @@ void log_client_none_connected(log_t *logdata)
 	}
 }
 
+void log_client_none_connected(log_t *logdata)
+{
+	logdata->connected = 0;
+
+	if (logdata->user->always_backlog)
+		return;
+
+	log_reinit_all(logdata);
+}
+
 void log_client_connected(log_t *logdata)
 {
 	mylog(LOG_DEBUG, "A client connected");
@@ -630,11 +665,14 @@ void log_client_connected(log_t *logdata)
 void log_advance_backlogs(log_t* ld, logfilegroup_t *lfg)
 {
 	int c;
-	(void)ld;
-	if (!conf_backlog || conf_backlog_lines == 0)
+
+	if (!lfg->track_backlog)
 		return;
 
-	if (lfg->skip_advance < conf_backlog_lines) {
+	if (!ld->user->backlog || ld->user->backlog_lines == 0)
+		return;
+
+	if (lfg->skip_advance < ld->user->backlog_lines) {
 		lfg->skip_advance++;
 		return;
 	}
@@ -679,6 +717,9 @@ int log_has_backlog(log_t *logdata, char *destination)
 	if (lfg->memlog)
 		return !list_is_empty(lfg->memlog);
 
+	if (!lfg->track_backlog)
+		return 0;
+
 	logfile_t *lf;
 	lf = list_get_first(&lfg->file_group);
 	if (lf != list_get_last(&lfg->file_group))
@@ -687,8 +728,6 @@ int log_has_backlog(log_t *logdata, char *destination)
 	return lf->backlog_offset != lf->len;
 }
 
-#define BOLD_CHAR 0x02
-#define LAMESTRING "!bip@bip.bip.bip PRIVMSG "
 
 /*
  * chan:
@@ -701,7 +740,7 @@ int log_has_backlog(log_t *logdata, char *destination)
  * 01-08-2005 10:46:11 < * jj!john@thebox.ofjj.net
  */
 
-char *log_beautify(char *buf, char *dest)
+char *log_beautify(log_t *logdata, char *buf, char *dest)
 {
 	int action = 0;
 	char *p;
@@ -728,8 +767,9 @@ char *log_beautify(char *buf, char *dest)
 		return _log_wrap(dest, buf);
 	lots = p - sots;
 	p++;
+
 	if (strncmp(p, "-!-", 3) == 0) {
-		if (conf_bl_msg_only)
+		if (logdata->user->bl_msg_only)
 			return NULL;
 		else
 			return _log_wrap(dest, buf);
@@ -786,7 +826,7 @@ char *log_beautify(char *buf, char *dest)
 				p++;
 
 		if (!p[0] || p[0] != ')' || !p[1] || p[1] != ':' ||
-				!p[2] || p[2] != ' ' || !p[3] || !p[4])
+				!p[2] || p[2] != ' ' || !p[3])
 			return _log_wrap(dest, buf);
 		p += 3;
 	} else {
@@ -815,7 +855,7 @@ char *log_beautify(char *buf, char *dest)
 
 	p = ret = (char *)malloc(
 		1 + lon + strlen(LAMESTRING) + lod + 2 + lots +	2 + lom + 3
-		+ action * (2 + strlen("ACTION ")) + out * strlen(" -> "));
+		+ action * (2 + strlen("ACTION ")) + out * strlen(PMSG_ARROW));
 	if (!p)
 		fatal("out of memory");
 
@@ -839,10 +879,10 @@ char *log_beautify(char *buf, char *dest)
 		p += strlen("ACTION ");
 	}
 	if (out) {
-		strcpy(p, " -> ");
-		p += strlen(" -> ");
+		strcpy(p, PMSG_ARROW);
+		p += strlen(PMSG_ARROW);
 	}
-	if (conf_backlog_no_timestamp == 0) {
+	if (logdata->user->backlog_no_timestamp == 0) {
 		memcpy(p, sots, lots);
 		p += lots;
 		*p++ = '>';
@@ -872,13 +912,16 @@ char *log_backread(log_t *logdata, char *destination, int *skip)
 
 	*skip = 0;
 
-	if (!conf_always_backlog && logdata->connected)
+	if (!logdata->user->always_backlog && logdata->connected)
 		return NULL;
 
 	lfg = hash_get(&logdata->logfgs, destination);
 	if (!lfg)
 		return NULL;
 
+	if (!lfg->track_backlog)
+		return NULL;
+
 	if (!logdata->backlogging) {
 		logdata->backlogging = 1;
 		mylog(LOG_DEBUG, "backlogging!");
@@ -899,8 +942,10 @@ char *log_backread(log_t *logdata, char *destination, int *skip)
 		return strdup(ptr);
 	}
 
-	if (!conf_log)
+	if (!conf_log) {
+		mylog(LOG_DEBUG, "No conf_log, not backlogging");
 		return NULL;
+	}
 
 	buf = (char *)malloc((LOGLINE_MAXLEN + 1) * sizeof(char));
 
@@ -934,7 +979,7 @@ next_file:
 		}
 		for(;;) {
 			c = fgetc(lf->file);
-			if (!conf_always_backlog)
+			if (!logdata->user->always_backlog)
 				lf->backlog_offset++;
 			if (c == EOF || c == '\n' || pos == LOGLINE_MAXLEN) {
 				/* change file if we reach EOF, if pos == maxlen
@@ -944,11 +989,11 @@ next_file:
 					mylog(LOG_DEBUG, "logline too long");
 				if (c == EOF || pos == LOGLINE_MAXLEN) {
 					mylog(LOG_DEBUGVERB, "EOF: %s (%d)!",
-							lf->filename,
-							conf_always_backlog);
+						lf->filename,
+						logdata->user->always_backlog);
 
 					list_it_next(&logdata->file_it);
-					if (!conf_always_backlog) {
+					if (!logdata->user->always_backlog) {
 						list_remove_first(
 							&lfg->file_group);
 						logfile_free(lf);
@@ -965,7 +1010,7 @@ next_file:
 					*skip = 1;
 					return buf;
 				}
-				ret = log_beautify(buf, destination);
+				ret = log_beautify(logdata, buf, destination);
 				if (ret == NULL) {
 					pos = 0;
 					continue;
@@ -985,24 +1030,25 @@ next_file:
 					"Error reading in logfile");
 		}
 		logdata->lastfile_seeked = 1;
-		mylog(LOG_DEBUG, "last file seedked!");
+		mylog(LOG_DEBUG, "last file seeked!");
 	}
 
 	c = fgetc(lf->file);
 	if (c == EOF) {
+		mylog(LOG_DEBUG, "Nothing more to backlog");
 		logdata->lastfile_seeked = 0;
 		logdata->backlogging = 0;
 		free(buf);
 		return NULL;
 	}
-	if (!conf_always_backlog)
+	if (!logdata->user->always_backlog)
 		lf->backlog_offset++;
 
 	if (c != '\n')
 		buf[pos++] = c;
 	for(;;) {
 		c = fgetc(lf->file);
-		if (!conf_always_backlog)
+		if (!logdata->user->always_backlog)
 			lf->backlog_offset++;
 		if (c == EOF || c == '\n' || pos == LOGLINE_MAXLEN) {
 			if (pos == LOGLINE_MAXLEN) {
@@ -1015,18 +1061,20 @@ next_file:
 				return NULL;
 			}
 
-			if (!conf_always_backlog && c == EOF)
+			if (!logdata->user->always_backlog && c == EOF)
 				lf->backlog_offset--;
 			buf[pos] = 0;
 			if (pos == 0) {
 				*skip = 1;
 				return buf;
 			}
-			ret = log_beautify(buf, destination);
+			ret = log_beautify(logdata, buf, destination);
 			if (ret == NULL) {
+				mylog(LOG_DEBUGVERB, "Line not backlogged");
 				pos = 0;
 				continue;
 			}
+			mylog(LOG_DEBUGVERB, "backlog: %s", ret);
 			free(buf);
 			return ret;
 		}
@@ -1063,10 +1111,10 @@ static int _log_write(log_t *logdata, logfilegroup_t *lfg, char *destination,
 	str[LOGLINE_MAXLEN] = 0;
 
 	if (lfg->memlog) {
-		char *r = log_beautify(str, destination);
+		char *r = log_beautify(logdata, str, destination);
 		if (r != NULL) {
 			list_add_last(lfg->memlog, r);
-			if (lfg->memc == conf_backlog_lines)
+			if (lfg->memc == logdata->user->backlog_lines)
 				free(list_remove_first(lfg->memlog));
 			else
 				lfg->memc++;
@@ -1085,7 +1133,7 @@ static int _log_write(log_t *logdata, logfilegroup_t *lfg, char *destination,
 	if (nbwrite != len + 1)
 		mylog(LOG_ERROR, "Error writing to %s logfile", lf->filename);
 	lf->len += nbwrite;
-	if (!logdata->connected || conf_always_backlog)
+	if (!logdata->connected || logdata->user->always_backlog)
 		log_advance_backlogs(logdata, lfg);
 	return nbwrite;
 }
@@ -1109,6 +1157,7 @@ void log_flush_all(void)
 	if (!log_all_logs)
 		return;
 
+	fflush(conf_global_log_file);
 	for (list_it_init(log_all_logs, &li); list_it_item(&li);
 			list_it_next(&li)) {
 		log_t *log = list_it_item(&li);
@@ -1127,14 +1176,14 @@ void log_flush_all(void)
 	}
 }
 
-log_t *log_new(char *user, char *network)
+log_t *log_new(struct user *user, char *network)
 {
 	log_t *logdata;
 
 	logdata = (log_t*)calloc(sizeof(log_t), 1);
 	if (!logdata)
 		fatal("out of memory");
-	logdata->user = strdup(user);
+	logdata->user = user;
 	logdata->network = strdup(network);
 	hash_init(&logdata->logfgs, HASH_NOCASE);
 	logdata->buffer = (char *)malloc((LOGLINE_MAXLEN + 1) * sizeof(char));
@@ -1144,7 +1193,29 @@ log_t *log_new(char *user, char *network)
 		fatal("out of memory");
 	logdata->connected = 0;
 	if (!log_all_logs)
-		log_all_logs = list_new(NULL);
+		log_all_logs = list_new(list_ptr_cmp);
 	list_add_last(log_all_logs, logdata);
 	return logdata;
 }
+
+void log_free(log_t *log)
+{
+	hash_iterator_t it;
+	logfilegroup_t *lfg;
+	logfile_t *lf;
+
+	list_remove(log_all_logs, log);
+
+	free(log->network);
+	free(log->buffer);
+
+	for (hash_it_init(&log->logfgs, &it); (lfg = hash_it_item(&it));
+			hash_it_next(&it)) {
+		log_reset(lfg);
+		if ((lf = list_remove_first(&lfg->file_group)))
+			logfile_free(lf);
+	}
+	hash_clean(&log->logfgs);
+	free(log);
+}
+
