@@ -2,7 +2,7 @@
  * $Id: log.h,v 1.26 2005/04/12 19:34:35 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,21 +46,23 @@ typedef struct logfilegroup
 	list_t *memlog;
 	int memc;
 	list_iterator_t backlog_it;
+	int track_backlog;
 } logfilegroup_t;
 
 typedef struct log {
 	hash_t logfgs;
 	char *network;
-	char *user;
 	char *buffer;
 	int connected;
 	int backlogging;
 	list_iterator_t file_it;
 	int lastfile_seeked;
+
+	struct user *user;
 } log_t;
 
 void log_close_all(log_t *logdata);
-log_t *log_new(char *user, char *network);
+log_t *log_new(struct user *user, char *network);
 void logdata_free(log_t *logdata);
 int log_compare_files(logfile_t *f1, char *f2);
 
@@ -93,4 +95,7 @@ int log_has_backlog(log_t *logdata, char *destination);
 void log_flush_all(void);
 void log_client_none_connected(log_t *logdata);
 void log_reset(logfilegroup_t *);
+int check_dir(char *filename, int is_fatal);
+void log_reinit_all(log_t *logdata);
+void log_free(log_t *log);
 #endif
