@@ -2,7 +2,7 @@
  * $Id: util.c,v 1.60 2005/04/12 19:34:35 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,6 +12,7 @@
  */
 
 #include "config.h"
+#include "connection.h"
 #include "util.h"
 #include <stdarg.h>
 #include <stdio.h>
@@ -24,7 +25,7 @@
 #include <errno.h>
 
 extern int conf_log_level;
-extern int conf_log;
+extern int conf_log_system;
 extern int errno;
 
 /*
@@ -41,7 +42,7 @@ int is_valid_nick(char *str)
 	while (*tmp != '\0' && (isalnum(*tmp) || *tmp == '-' || *tmp == '[' ||
 			*tmp == ']' || *tmp == '\\' || *tmp == '`' ||
 			*tmp == '^' || *tmp == '{' || *tmp == '}' ||
-			*tmp == '|'))
+			*tmp == '|' || *tmp == '_' ))
 		tmp++;
 	return (*tmp == '\0');
 }
@@ -74,12 +75,44 @@ char *timestamp(void)
 	return ts;
 }
 
+char *hrtime(time_t s)
+{
+	static char ts[20];
+	struct tm *tm;
+
+	if (s == 0)
+		return "never";
+	tm = localtime(&s);
+
+	snprintf(ts, 20, "%02d-%02d-%04d %02d:%02d:%02d", tm->tm_mday,
+			tm->tm_mon + 1, tm->tm_year + 1900, tm->tm_hour,
+			tm->tm_min, tm->tm_sec);
+	return ts;
+}
+
+#ifdef HAVE_LIBSSL
+char *checkmode2text(int v)
+{
+	switch (v) {
+	case SSL_CHECK_BASIC:
+		return "basic";
+	case SSL_CHECK_CA:
+		return "ca";
+	default:
+		return "none";
+	}
+}
+#endif
+
 extern FILE *conf_global_log_file;
 
 void _mylog(int level, char *fmt, va_list ap)
 {
 	char *prefix;
 
+	if (!conf_log_system)
+		return;
+
 	if (level > conf_log_level)
 		return;
 
@@ -88,7 +121,7 @@ void _mylog(int level, char *fmt, va_list ap)
 			prefix = "FATAL: ";
 			break;
 		case LOG_DEBUGVERB:
-			prefix = "DEBUG: ";
+			prefix = "DEBUGVERB: ";
 			break;
 		case LOG_DEBUG:
 			prefix = "DEBUG: ";
@@ -119,9 +152,6 @@ void mylog(int level, char *fmt, ...)
 {
 	va_list ap;
 
-	if (!conf_log)
-		return;
-
 	va_start(ap, fmt);
 	_mylog(level, fmt, ap);
 	va_end(ap);
@@ -188,6 +218,9 @@ static struct list_item *list_item(void *ptr)
 void list_add_first(list_t *list, void *ptr)
 {
 	struct list_item *li;
+
+	if (!ptr)
+		fatal("Cannot add NULL ptr to list.");
 	li = list_item(ptr);
 	if (!list->first) {
 		list->first = list->last = li;
@@ -200,6 +233,8 @@ void list_add_first(list_t *list, void *ptr)
 
 void list_add_first_uniq(list_t *list, void *ptr)
 {
+	if (!ptr)
+		fatal("Cannot add NULL ptr to list.");
 	if (list_get(list, ptr))
 		return;
 	list_add_first(list, ptr);
@@ -208,6 +243,9 @@ void list_add_first_uniq(list_t *list, void *ptr)
 void list_add_last(list_t *list, void *ptr)
 {
 	struct list_item *li;
+
+	if (!ptr)
+		fatal("Cannot add NULL ptr to list.");
 	li = list_item(ptr);
 	if (!list->first) {
 		list->first = list->last = li;
@@ -262,28 +300,6 @@ void *list_remove_last(list_t *list)
 	return ptr;
 }
 
-/*
-static void *list_remove_item(list_t *l, struct list_item *li)
-{
-	void *ret = li->ptr;
-	if (!li->prev) {
-		if (l->first != li)
-			fatal("list_remove_item");
-		l->first = li->next;
-	} else
-		li->prev->next = li->next;
-
-	if (!li->next) {
-		if (l->last != li)
-			fatal("list_remove_item");
-		l->last = li->prev;
-	} else
-		li->next->prev = li->prev;
-	free(li);
-	return ret;
-}
-*/
-
 void *list_remove_if_exists(list_t *list, void *ptr)
 {
 	list_iterator_t li;
@@ -337,12 +353,19 @@ void list_it_init(list_t *list, list_iterator_t *ti)
 {
 	ti->list = list;
 	ti->cur = list->first;
+	ti->next = NULL;
 }
 
 void list_it_next(list_iterator_t *ti)
 {
-	if (ti->cur)
+	if (ti->cur) {
+		if (ti->next)
+			fatal("list_it_next: inconsistent interator state");
 		ti->cur = ti->cur->next;
+	} else if (ti->next) {
+		ti->cur = ti->next;
+		ti->next = NULL;
+	}
 }
 
 void *list_it_item(list_iterator_t *ti)
@@ -369,7 +392,8 @@ void *list_it_remove(list_iterator_t *li)
 
 	void *ptr = li->cur->ptr;
 	struct list_item *item = li->cur;
-	li->cur = li->cur->next;
+	li->next = li->cur->next;
+	li->cur = NULL;
 	free(item);
 	return ptr;
 }
@@ -411,6 +435,7 @@ static int hash_item_cmp(struct hash_item *a, char *b)
 void hash_init(hash_t *h, int options)
 {
 	int i;
+
 	memset(h, 0, sizeof(hash_t));
 	for (i = 0; i < 256; i++) {
 		switch (options) {
@@ -431,9 +456,13 @@ void hash_init(hash_t *h, int options)
 void hash_clean(hash_t *h)
 {
 	int i;
+	struct hash_item *hi;
+
 	for (i = 0; i < 256; i++) {
-		while (list_remove_first(&h->lists[i]))
-			;
+		while ((hi = list_remove_first(&h->lists[i]))) {
+			free(hi->key);
+			free(hi);
+		}
 	}
 }
 
@@ -509,50 +538,73 @@ void *hash_remove(hash_t *hash, char *key)
 	return ptr;
 }
 
+int hash_is_empty(hash_t *h)
+{
+	int i;
+
+	for (i = 0; i < 256; i++) {
+		if (!list_is_empty(&h->lists[i]))
+			return 0;
+	}
+	return 1;
+}
+
 void hash_it_init(hash_t *h, hash_iterator_t *hi)
 {
 	memset(hi, 0, sizeof(hash_iterator_t));
 	hi->hash = h;
 
-	while (list_is_empty(&h->lists[hi->list]) && hi->list < 256)
+	while (hi->list < 256 && list_is_empty(&h->lists[hi->list]))
 		hi->list++;
 	if (hi->list < 256)
-		hi->cur = h->lists[hi->list].first;
-	else
-		hi->cur = NULL;
+		list_it_init(&h->lists[hi->list], &hi->lit);
 }
 
 void hash_it_next(hash_iterator_t *hi)
 {
-	hash_t *hash = hi->hash;
-
-	hi->cur = hi->cur->next;
-	while (!hi->cur) {
-		hi->list++;
-		if (hi->list == 256) {
-			hi->cur = NULL;
-			return;
-		}
-		hi->cur = hash->lists[hi->list].first;
+	list_it_next(&hi->lit);
+	if (!list_it_item(&hi->lit)) {
+		do {
+			hi->list++;
+			if (hi->list == 256)
+				return;
+		} while (list_is_empty(&hi->hash->lists[hi->list]));
+		list_it_init(&hi->hash->lists[hi->list], &hi->lit);
 	}
 }
 
 void *hash_it_item(hash_iterator_t *h)
 {
-	if (!h->cur)
+	struct hash_item *hi;
+
+	hi = list_it_item(&h->lit);
+	if (!hi)
 		return NULL;
-	struct hash_item *hi = h->cur->ptr;
 	return hi->item;
 }
 
 char *hash_it_key(hash_iterator_t *h)
 {
-	if (!h->cur)
+	struct hash_item *hi;
+	hi = list_it_item(&h->lit);
+	if (!hi)
 		return NULL;
-	struct hash_item *hi = h->cur->ptr;
 	return hi->key;
 }
 
+void *hash_it_remove(hash_iterator_t *hi)
+{
+	struct hash_item *hitem;
+	void *ptr;
+
+	hitem = list_it_remove(&hi->lit);
+
+	ptr = hitem->item;
+	free(hitem->key);
+	free(hitem);
+	return ptr;
+}
+
 void hash_dump(hash_t *h)
 {
 	hash_iterator_t it;
@@ -579,4 +631,3 @@ int ischannel(char p)
 {
 	return (p == '#' || p == '&' || p == '+' || p == '!');
 }
-
